#!/usr/bin/env python

"""
pykde-latest.py
Shows the web page showing the latest messages from the PyKDE mailing list.
This module can be used with the PyAutomation KPart plugin for Konqueror.

(C) David Boddie 2004

This software is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public
License along with this library; see the file COPYING
If not, write to the Free Software Foundation, Inc.,
59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
"""

from qt import SIGNAL

from kdecore import KURL

from khtml import DOM

# Import Paul's khtmldom interface to KHTML for a more Pythonic DOM
# experience.

import khtmldom

import sys


# Define an exception which is raised when an htmlError signal is
# received from the KHTMLPart.

class HTMLError(Exception):

    pass


def html_error(*args):

    raise HTMLError, "htmlError(%i, %s, %s)" % \
        (errorCode, text, reqUrl.url())


def _main(part):

    # Connect a signal from the part to a function which raises an
    # exception.
    part.connect(
        part, SIGNAL("htmlError(int, const QString&, const KURL&)"),
        html_error
        )
    
    try:
    
        automate(part)
    
    except HTMLError:
    
        show_error_page(part, "Failed to open the URL.")
        return


# Ideally, we'd use the above arrangement to catch page misses but, because
# the htmlError signal is protected, we need to perform various sanity
# checks throughout.

def main(part):

    global use_part
    
    use_part = part
    
    # Open the PyKDE mailing list page.
    part.connect(part, SIGNAL("completed()"), continue1)
    part.openURL(KURL("http://mats.imk.fraunhofer.de/pipermail/pykde/"))

def continue1():

    part = use_part
    part.disconnect(part, SIGNAL("completed()"), continue1)

    # Retrieve the document object.
    document = part.document()
    
    # Obtain a wrapped form of the document.
    document = khtmldom.Node(document)
    
    # Find the document body.
    bodies = find_nodes(document, "BODY", case = 0)
    
    print bodies
    
    try:
    
        body = bodies[0]
    
    except IndexError:
    
        show_error_page(part, "Failed to find a document body.")
        return
    
    # Find all tables in the body.
    tables = find_nodes(body, "TABLE", case = 0)
    
    # Find the first table in the document.
    try:
    
        table = tables[0]
    
    except IndexError:
    
        show_error_page(part, "Failed to find a table of archives.")
        return
    
    # The second row of the table contains links to the most recent month.
    try:
    
        row = find_nodes(table, "TR", case = 0)[1]
    
    except IndexError:
    
        show_error_page(part, "Failed to find the latest archive.")
        return
    
    # Retrieve the links to the forms for viewing the most recent monthly
    # archive.
    links = find_nodes(row, "A", case = 0)
    
    # We are interested in a chronological listing of messages.
    try:
    
        link = links[3]
    
    except IndexError:
    
        show_error_page(part, "Failed to find the link to the messages.")
        return
    
    # Find the HREF attribute in the attributes for the link.
    try:
    
        href = link.attributes[(u"HREF")]
    
    except KeyError:
    
        show_error_page(part, "Failed to find the URL for the message archive.")
        return
    
    # Finally, retrieve the URL for the messages listing.
    url = href.nodeValue
    
    # Open the URL in the browser.
    part.connect(part, SIGNAL("completed()"), continue2)
    part.openURL(KURL("http://mats.imk.fraunhofer.de/pipermail/pykde/" + url))

def continue2():

    part = use_part
    part.disconnect(part, SIGNAL("completed()"), continue2)
    
    part.view().ensureVisible(0, part.view().contentsHeight())
    
    del part


def show_error_page(part, message):

    # Clear the document and replace it with some information.
    
    part.begin()
    
    part.write( "\n".join(
        [
            "<html>",
            "<head>",
            "  <title>Message from script</title>",
            '  <meta http-equiv="Content-Type" content="text/html">\n',
            "</head>",
            "",
            "<body>",
            "<h1>Message from script</h1>",
            "",
            message,
            "",
            "</body>",
            "</html>",
            ""
        ]
        ) )
    
    part.end()


def find_nodes(node, name, case = 1):

    # Find all nodes with the given name under the named node in the
    # document, respecting case sensitivity if case is set to one.
    
    found = []
    
    for child in node.childNodes:
    
        # Check this node to determine whether it matches the name
        # given.
        
        if case == 1 and child.nodeName == name:
        
            found.append(child)
        
        elif case == 0 and child.nodeName.lower() == name.lower():
        
            found.append(child)
        
        # Check this node's children, too.
        
        found = found + find_nodes(child, name, case)
    
    return found

def html_repr(text):

    new = []
    
    for c in text:
    
        if c == "&":
            new.append("&amp;")
        elif c == "<":
            new.append("&lt;")
        elif c == ">":
            new.append("&gt;")
        else:
            new.append(c)
    
    return "".join(new)
