#!/usr/bin/env python

"""
pyautomation.py

KPart plugin for KHTML to load scripts for automation purposes.

Copyright (C) 2003-5 David Boddie <david@boddie.org.uk>

This software is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public
License along with this library; see the file COPYING
If not, write to the Free Software Foundation, Inc.,
59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
"""

#from debugtools import debug, trap

# Import the required Qt and KDE classes.

import sip
from qt import QCheckBox, QColor, QHBox, QLabel, QMessageBox, QString, \
               SIGNAL, Qt, QTextBrowser, QWidget, QVBoxLayout

from kdecore import i18n, KAccel, KInstance, KLibFactory, KShortcut, KURL
from kparts import KParts
from kdeui import KAction, KPushButton, KSeparator
from kfile import KURLRequester
from khtml import KHTMLPart, DOM
from kio import KRun

import imp, os, sys, traceback


# Define a class which will be initialised when the plugin is loaded.

class FactoryClass(KLibFactory):

    instance = None
    
    def __init__(self, parent = None, name = None):
    
        #print "FactoryClass.__init__%s" % repr((parent, name))
        
        KLibFactory.__init__(self, parent, name)
        
        if FactoryClass.instance is None:
        
            FactoryClass.instance = KInstance("PyAutomation")
    
    #__init__ = trap(__init__)
    
    def createObject(self, parent = None, name = None, class_name = "QObject",
                     args = None):
    
        if args is None:
        
            args = []
        
        #print "FactoryClass.createObject%s" % repr((parent, name, class_name, args))
        
        plugin = PyAutomation(parent, name)
        
        #print "created %s" % repr(plugin)
        
        # Either of these appear to work:
        #self.emit(SIGNAL("objectCreated(KParts::Plugin)"), (self.plugin,))
        self.emit(SIGNAL("objectCreated(QObject)"), (plugin,))
        
        #print "returning"
        return plugin
    
    #createObject = trap(createObject)
    create = createObject


# Define the plugin class, derived from the standard KParts.Plugin class.

class PyAutomation(KParts.Plugin):

    def __init__(self, parent = None, name = None):
    
        KParts.Plugin.__init__(self, parent, name)
        
        #self.name = name
        #self.parent = parent
        self.user_module = None
        self.user_module_path = None
        
        self.error_window = None
        
        actions = self.actionCollection()
        
        #print actions
        
        self.action = KAction(
            "Automate the browser",
            "pyautomation",
            KShortcut.null(),
            self.slotOpenDialog,
            actions,
            "pyautomation-dialog"
            )
    
    #__init__ = trap(__init__)
    
    def slotOpenDialog(self):
    
        # The parent is assumed to be a KHTMLPart
        
        if not self.parent().inherits("KHTMLPart"):
        
            title = i18n( "Cannot automate the browser" )
            text = i18n( "You can only automate web pages." )
            
            QMessageBox.warning( None, title, text )
            return
        
        # Open a window to use to specify the search keyword/pattern.
        self.dialog = ScriptDialog(self.execute_script, self.edit_script)
        self.dialog.show()
    
    def execute_script(self):
    
        # Check that the parent still exists.
        try:
        
            parent = self.parent()
        
        except RuntimeError:
        
            title = i18n( "Cannot execute script" )
            text = i18n( "The original document is no longer available." )
            
            QMessageBox.warning( None, title, text )
            return
        
        # The parent is assumed to be a KHTMLPart
        
        if not parent.inherits("KHTMLPart"):
        
            title = i18n( "Cannot execute script" )
            text = i18n( "You can only automate the browser from within "
                         "web pages." )
            
            QMessageBox.warning( None, title, text )
            return
        
        # Obtain the path to a user script from the script dialog.
        
        path = self.dialog.locate_script()
        
        if path is None:
        
            title = i18n( "Cannot execute script" )
            text = i18n( "Please supply a Python script." )
            
            QMessageBox.warning( None, title, text )
            return
        
        # Use the cast function to ensure that we can access the parent
        # part fully.
        
        #new_part = cast.cast(parent, KHTMLPart)
        new_part = parent
        print
        print "-"*50
        print new_part
        print "-"*50
        print
        
        # Try to import the user's script.
        
        try:
        
            code = compile(open(path).read(), path, "exec")
            user_module = imp.new_module("__user__")
            
            exec code in user_module.__dict__
            user_module_path = path
        
        except:
        
            title = i18n( "Cannot execute script" )
            
            lines = traceback.format_exception( *( sys.exc_info() ) )
            
            self.open_error_window(title, lines, path)
            return
        
        # Record the user module and its path.
        if self.user_module is not None and \
            self.user_module.__dict__.has_key("shutdown"):
        
            # Close down the old module.
            self.user_module.shutdown()
        
        self.user_module = user_module
        self.user_module_path = user_module_path
        
        # Pass the user's interface object and the part object to the
        # user's script.
        
        try:
        
            self.user_module.main(new_part)
        
        except:
        
            title = i18n( "Cannot execute script" )
            
            lines = traceback.format_exception( *( sys.exc_info() ) )
            
            self.open_error_window(title, lines, path)
            return
    
    def edit_script(self):
    
        path = self.dialog.locate_script()
        
        if path is None:
        
            title = i18n( "Cannot edit script" )
            text = i18n( "Please supply a Python script." )
            
            QMessageBox.warning( None, title, text )
            return
        
        # Launch an application to edit the script.
        self.user_module_temp = KRun(KURL(path))
    
    def open_error_window(self, title, lines, path):
    
        if self.error_window is None or \
            self.error_window.path != path:
        
            self.error_window = ErrorInfo(title, "\n".join(lines), path)
        
        else:
        
            self.error_window.add_text("\n".join(lines))
        
        self.error_window.show()


class ScriptDialog(QWidget):

    def __init__(self, slotExecute, slotEdit):
    
        QWidget.__init__(self)
        
        # The path is initially unset.
        self.path = None
        
        self.setCaption( i18n("Select a script to execute") )
        
        # Create a vertical layout.
        layout = QVBoxLayout(self, 2, 4)
        
        # Create a label.
        label = QLabel(self)
        label.setText( i18n("Script to use:") )
        
        # Create a widget to contain the URL of the script to execute.
        self.url_request = KURLRequester(self)
        self.url_request.lineEdit().setReadOnly(1)
        
        self.url_request.setFilter("*.py")
        self.connect(
            self.url_request, SIGNAL("textChanged( const QString & )"),
            self.manage_buttons
            )
        self.connect(
            self.url_request, SIGNAL("urlSelected( const QString & )"),
            self.manage_buttons
            )
        
        self.edit_button = KPushButton(i18n("Edit script"), self)
        
        self.connect(
            self.edit_button, SIGNAL("clicked()"), slotEdit
            )
        
        # Add a separator between the script details and the following
        # action push buttons.
        separator = KSeparator(self)
        
        # Add OK and Cancel buttons.
        hbox = QHBox(self)
        
        self.ok_button = KPushButton(i18n("OK"), hbox)
        self.cancel_button = KPushButton(i18n("Cancel"), hbox)
        
        self.connect(
            self.ok_button, SIGNAL("clicked()"), slotExecute
            )
        
        self.connect(
            self.cancel_button, SIGNAL("clicked()"), self.close
            )
        
        self.connect(
            self.url_request, SIGNAL("returnPressed()"),
            self.ok_button.animateClick
            )
        
        # Add the widgets to the layout.
        layout.addWidget(label)
        layout.addWidget(self.url_request)
        layout.addWidget(self.edit_button)
        layout.addWidget(separator)
        layout.addWidget(hbox)
        
        # Use an accelerator to add the ability to cancel the window
        # by pressing the Escape key.
        
        self.accel = KAccel(self)
        
        self.accel.insertItem( i18n("Cancel"), "Cancel", "Escape" )
        self.accel.connectItem( "Cancel", self.close )
        
        # Modify the buttons according to the state of the dialog.
        self.manage_buttons()
    
    def close(self, alsoDelete = 1):
    
        QWidget.close(self, alsoDelete)
    
    def locate_script(self):
    
        # Retrieve the URL describing the location of the script, assuming
        # that it corresponds to a local file.
        self.path = str(self.url_request.url())
        
        if not os.path.exists(self.path):
        
            self.path = None
        
        return self.path
    
    def manage_buttons(self, path = None):
    
        # If this slot is called with an argument then store that and use
        # it to determine whether some buttons need to be modified.
        
        if path is not None:
        
            self.path = str(path)
        
        # If there is no path set then shade the OK and Edit buttons.
        enabled = (self.path is not None)
        
        self.ok_button.setEnabled(enabled)
        self.edit_button.setEnabled(enabled)
        self.ok_button.update()


class ErrorInfo(QWidget):

    def __init__(self, caption, info, path):
    
        QWidget.__init__(self)
        
        self.setCaption(caption)
        
        # Record the current raw text displayed.
        self.info = info
        
        # Record the path to the script.
        self.path = path
        
        # Create a vertical layout.
        layout = QVBoxLayout(self)
        
        # Create a label.
        label = QLabel(self)
        label.setText(
            i18n("Output")
            )
        
        self.info_widget = QTextBrowser(self)
        self.info_widget.setTextFormat(Qt.RichText)
        self.info_widget.setText(self.prepare_text(info))
        
        hbox = QHBox(self)
        self.cancel_button = KPushButton(i18n("Cancel"), hbox)
        
        self.connect(
            self.cancel_button, SIGNAL("clicked()"), self.close
            )
        
        # Add the widgets to the layout.
        layout.addWidget(label)
        layout.addWidget(self.info_widget)
        layout.addWidget(hbox)
    
    def prepare_text(self, text):
    
        new = ["<pre>"]
        
        for c in text:
        
            if c == "&":
                new.append("&amp;")
            elif c == "<":
                new.append("&lt;")
            elif c == ">":
                new.append("&gt;")
            else:
                new.append(c)
        
        new.append("</pre>")
        
        return "".join(new)
    
    def add_text(self, text):
    
        pieces = [self.info, "\n\n", text]
        
        self.info = "".join(pieces)
        
        self.info_widget.setText(self.prepare_text(self.info))
