#!/usr/bin/env python

"""
pyvalidatorplugin.py

KPart plugin for KHTML based on the C++ example at

http://developer.kde.org/documentation/tutorials/dot/writing-plugins.html

Copyright (C) 2003 David Boddie <david@boddie.org.uk>

This software is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public
License along with this library; see the file COPYING
If not, write to the Free Software Foundation, Inc.,
59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

"""

#from debugtools import debug, trap
#import kpartdebug

# Import the required Qt and KDE classes.

try:

    import sip
    from qt import QMessageBox, QString, SIGNAL
    
    from kdecore import i18n, KInstance, KLibFactory, KShortcut, KURL
    from kparts import KParts
    from kdeui import KAction
    from khtml import KHTMLPart

except:

    import sys, traceback
    
    traceback.print_exception(
        *(sys.exc_info()), **{"file": open("/tmp/pydebug.txt", "a")}
        )

# Define a class which will be initialised when the plugin is loaded.

class FactoryClass(KLibFactory):

    instance = None
    
    def __init__(self, parent = None, name = None):
    
        #print "FactoryClass.__init__%s" % repr((parent, name))
        
        KLibFactory.__init__(self, parent, name)
        
        if FactoryClass.instance is None:
        
            FactoryClass.instance = KInstance("PyValidatorPlugin")
    
    def createObject(self, parent = None, name = None, class_name = "QObject",
                     args = None):
    
        if args is None:
        
            args = []
        
        #print "FactoryClass.createObject%s" % repr((parent, name, class_name, args))
        
        plugin = PyValidatorPlugin(parent, name)
        
        #print "created %s" % repr(plugin)
        
        # Either of these appear to work:
        #self.emit(SIGNAL("objectCreated(KParts::Plugin)"), (self.plugin,))
        self.emit(SIGNAL("objectCreated(QObject)"), (plugin,))
        
        #print "returning"
        return plugin
    
    create = createObject


# Define the plugin class, derived from the standard KParts.Plugin class.

class PyValidatorPlugin(KParts.Plugin):

    def __init__(self, parent = None, name = None):
    
        #print "PyValidatorPlugin.__init__%s" % repr((parent, name))
        KParts.Plugin.__init__(self, parent, name)
        
        #self.name = name
        #self.parent = parent
        
        actions = self.actionCollection()
        
        #print actions
        
        self.action = KAction(
            "&Python Validate Web Page (plugin)",
            "pyvalidatorplugin",
            KShortcut.null(),
            self.slotValidate,
            actions,
            "pythonvalidatewebpage"
            )
    
    def slotValidate(self):
    
        # The parent is assumed to be a KHTMLPart
        
        if not self.parent().inherits("KHTMLPart"):
        
            title = i18n( "Cannot validate source" )
            text = i18n( "You cannot validate anything except web pages with\n"
                         "this plugin, sorry." )
            
            QMessageBox.warning( None, title, text )
            
        else:
        
            part = self.parent()
            
            #print part
            
            # Get URL
            url = part.url()
            w3c = self.validateURL( url )
            
            if isinstance(w3c, KURL):
            
                part.openURL( w3c )
    
    def validateURL(self, url):
    
        # Check syntax
        if url.isMalformed():
        
            title = i18n("Malformed URL")
            text = i18n("<qt><p>The URL you entered is not valid, please "
                        "correct it and try again</p></qt>")
            QMessageBox.warning( None, title, text )
            return QString.null
        
        # Create w3c URL
        w3c = KURL("http://validator.w3.org/check/")
        
        # Set entered URL as a parameter
        q = url.url()
        q = str(KURL.encode_string( q ))
        p = "uri="
        p = p + q
        
        w3c.setQuery( p )
        
        return w3c # Check syntax
